@@ -13,7 +13,7 @@ use Validation::Class::Util;
 
 use overload bool => \&validate, '""' => \&messages, fallback => 1;
 
-our $VERSION = '7.900050'; # VERSION
+our $VERSION = '7.900051'; # VERSION
 
 
 
@@ -170,7 +170,7 @@ Validation::Class::Simple::Streamer - Simple Streaming Data Validation
 
 =head1 VERSION
 
-version 7.900050
+version 7.900051
 
 =head1 SYNOPSIS
 
@@ -228,9 +228,9 @@ derived from the L<Validation::Class::Simple> class.
 
 If you are new to Validation::Class, or would like more information on
 the underpinnings of this library and how it views and approaches
-data validation, please review L<Validation::Class::Whitepaper>.
-Please review the L<Validation::Class::Simple/GUIDED-TOUR> for a detailed
-step-by-step look into how Validation::Class works.
+data validation, please review L<Validation::Class::Whitepaper>. Please review
+the L<Validation::Class::Cookbook/GUIDED-TOUR> for a detailed step-by-step look
+into how Validation::Class works.
 
 =head1 METHODS
 
